// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_authentication_bloc/src/domain/entities/account.dart';

/// {@template authentication_remote_data_source}
/// A remote data source for authentication.
/// It is responsible for all the external communication with the authentication
/// providers.
/// {@endtemplate}
abstract class AuthenticationRemoteDataSource<Data>
    extends BaseRemoteDataSource {
  /// {@macro authentication_remote_data_source}
  const AuthenticationRemoteDataSource();

  // SignUp/SignIn methods ====================================================

  /// Sign up with email and password.
  Future<Account> signUpWithEmailAndPassword({
    required String email,
    required String password,
  });

  /// Sign in with email and password.
  Future<Account> signInWithEmailAndPassword({
    required String email,
    required String password,
  });

  /// Sign in anonymously.
  Future<Account> signInAnonymously();

  /// Sign in with Google.
  Future<Account> signInWithGoogle();

  /// Sign out.
  Future<void> signOut();

  // Account management methods ===============================================

  /// Refresh the current account.
  Future<Account> refresh();

  /// Reauthenticate the current account.
  Future<Account> reauthenticate();

  /// Update the current account's email.
  Future<Account> updateEmail({required String email});

  /// Update the current account's password.
  Future<Account> updatePassword({required String password});

  /// Delete the current account.
  Future<void> delete();

  // Email related stuff ======================================================

  /// Send an email verification.
  Future<void> sendEmailVerification();

  /// Send a password reset email.
  Future<void> sendPasswordResetEmail({required String email});

  /// Confirm password reset.
  Future<void> confirmPasswordReset({
    required String code,
    required String newPassword,
  });

  /// Verify password reset code.
  Future<bool> verifyPasswordResetCode({required String code});
}
